function [pas,diff1,diff2,diffinf] = erreur(eq,tfinal,h0,niter,schema)

%
% function [pas,diff1,diff2,diffinf] = erreur(eq,tfinal,h0,niter,schema)
%
% Calcule l erreur commise lors de l integration de l edp
%
% pas = pas de temps, diff* = erreur commise lors de la resolution
% eq = probleme considere, tfinal = temps final, h0 = pas de temps
% initial, niter = nombre de calculs d erreurs, schema = schema
% d integration
%

% erreur(1,1,0.5,10,1) essai
% erreur(2,5,0.25,8,1) vdp
% erreur(3,5,1,8,1) lorenz
% erreur(4,20,1,8,1) euler

if schema == 1
 r = 2;
elseif schema == 3
 r = 4;
else
 fprintf('schema non defini')
 break
end

h = h0;
n = tfinal/h;

str1 = cat(2,'pb',num2str(eq)); % probleme
str2 = cat(2,'n',num2str(n)); % niter
str3 = cat(2,'h',num2str(h)); % deltat
str4 = cat(2,'sch',num2str(schema)); % schema
str = cat(2,str1,str2);
str = cat(2,str,str3);
str = cat(2,str,str4);
strb = cat(2,str,'.sol');
fid = fopen(strb,'r');
sol2 = fread(fid,'float64');
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Lecture des integrations successives du probleme
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k = 1:niter
 n = tfinal/h;
 sol1 = sol2;
 clear sol2

 str1 = cat(2,'pb',num2str(eq)); % probleme
 str2 = cat(2,'n',num2str(2*n)); % niter
 str3 = cat(2,'h',num2str(h/2)); % deltat
 str4 = cat(2,'sch',num2str(schema)); % schema
 str = cat(2,str1,str2);
 str = cat(2,str,str3);
 str = cat(2,str,str4);
 strb = cat(2,str,'.sol');
 fid = fopen(strb,'r');
 sol2 = fread(fid,'float64');
 fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Calcul de l erreur
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 if eq == 1
  for i = 1:n+1
   ecart(1,i) = sol1((i-1)*r+1)-sol2(2*(i-1)*r+1);
   diff(i) = sqrt((ecart(1,i))^2);
  end
 elseif eq == 2
  for i = 1:n+1
   ecart(1,i) = sol1((i-1)*2*r+1)-sol2(2*(i-1)*2*r+1);
   ecart(2,i) = sol1((i-1)*2*r+r+1)-sol2(2*(i-1)*2*r+r+1);
   diff(i) = sqrt((ecart(1,i))^2+(ecart(2,i))^2);
  end
 elseif eq == 3
  for i = 1:n+1
   ecart(1,i) = sol1((i-1)*3*r+1)-sol2(2*(i-1)*3*r+1);
   ecart(2,i) = sol1((i-1)*3*r+r+1)-sol2(2*(i-1)*3*r+r+1);
   ecart(3,i) = sol1((i-1)*3*r+2*r+1)-sol2(2*(i-1)*3*r+2*r+1);
   diff(i) = sqrt((ecart(1,i))^2+(ecart(2,i))^2+(ecart(3,i))^2);
  end
 elseif eq == 4
  for i = 1:n+1
   ecart(1,i) = sol1((i-1)*3*r+1)-sol2(2*(i-1)*3*r+1);
   ecart(2,i) = sol1((i-1)*3*r+r+1)-sol2(2*(i-1)*3*r+r+1);
   ecart(3,i) = sol1((i-1)*3*r+2*r+1)-sol2(2*(i-1)*3*r+2*r+1);
   diff(i) = sqrt((ecart(1,i))^2+(ecart(2,i))^2+(ecart(3,i))^2);
  end
 else
  fprintf('Probleme non defini')
  break
 end

 diff1(k) = h*norm(diff,1);
 diff2(k) = h*norm(diff,2);
 diffinf(k) = h*norm(diff,inf);

 pas(k) = h;
 h = h/2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Trace log-log de l erreur en fonction du pas de temps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

loglog(pas,diff1)
xlabel('Pas de temps')
ylabel('Erreur, en norme 1')
if eq == 1
 title('Probleme y'' = - y, y(0) = 1, sur [0,1]')
elseif eq == 2
 title('Probleme de Van de Pol, avec CI x(0) = - 0.5 et y(0) = 0')
elseif eq == 3
 title('Systeme de Lorenz, avec CI x(0) = - 8, y(0) = 8 et z(0) = 27')
else
 title('Angles d''Euler, avec CI y1(0) = 1, y2(0) = 0 et y3(0) = 0.9')
end
fprintf('Norme 1\n')

pause

loglog(pas,diff2)
xlabel('Pas de temps')
ylabel('Erreur, en norme 2')
if eq == 1
 title('Probleme y'' = - y, y(0) = 1, sur [0,1]')
elseif eq == 2
 title('Probleme de Van de Pol, avec CI x(0) = - 0.5 et y(0) = 0')
elseif eq == 3
 title('Systeme de Lorenz, avec CI x(0) = - 8, y(0) = 8 et z(0) = 27')
else
 title('Angles d''Euler, avec CI y1(0) = 1, y2(0) = 0 et y3(0) = 0.9')
end
fprintf('Norme 2\n')

pause

loglog(pas,diffinf)
xlabel('Pas de temps')
ylabel('Erreur, en norme sup')
if eq == 1
 title('Probleme y'' = - y, y(0) = 1, sur [0,1]')
elseif eq == 2
 title('Probleme de Van de Pol, avec CI x(0) = - 0.5 et y(0) = 0')
elseif eq == 3
 title('Systeme de Lorenz, avec CI x(0) = - 8, y(0) = 8 et z(0) = 27')
else
 title('Angles d''Euler, avec CI y1(0) = 1, y2(0) = 0 et y3(0) = 0.9')
end
fprintf('Norme infinie\n')
